// TestCaseModel
// (c)2007 Mark Bulmahn. mBulmahn@web.de
// License: GPL 2.0
// Project: TestCaseGenerator. http://www.codeplex.com/TestCaseGenerator

using System;
using System.Data;
using System.Collections.Generic;
using System.Text;
using SerializationHelperLib;
using PICTWrapper;

namespace TestCaseModel
{
    /// <summary>
    /// The test model is where everything comes together ...
    /// It represents a test scenario which in turn consists of these elements:
    /// Parameters and equivalent classes, expected results, result rules and
    /// a data table for the test cases.
    /// From here the automatic generation of test cases will be initiated.
    /// At the same time it is the persistence layer for saving and loading
    /// models to and from disk by inheriting serialization functions from
    /// XmlPersistenceBase.
    /// </summary>
    [Serializable]
    public class TestModel : XmlPersistenceBase<TestModel>
    {
        private static Dictionary<string, TestModel> instances = new Dictionary<string, TestModel>();

        public const string COLTYPE = "ColumnType";
        public const string COLTYPE_PARAMETER = "Parameter";
        public const string COLTYPE_RESULT = "ExpectedResult";
        public const string COLTYPE_ID = "IdColumn";

        ITcGenerator generator = null;
        TestCaseDataTable testCasesTemp = null;

        public static TestModel GetInstance(string sessionId)
        {
            return instances[sessionId];
        }

        public static TestModel GetNewInstance(string sessionId, string fileName)
        {
            TestModel newInst = TestModel.CreateFromXml(fileName);
            instances.Add(sessionId, newInst);
            return newInst;
        }

        TestCaseDataTable testCases = null;
        public TestCaseDataTable TestCases
        {
            get { return testCases; }
            set { testCases = value; }
        }

        Parameters testParameters = new Parameters();
        public Parameters TestParameters
        {
            get { return testParameters; }
            set { testParameters = value; }
        }

        string testDescription = "";
        public string TestDescription
        {
            get { return testDescription; }
            set { testDescription = value; }
        }

        ExpectedResults testResults = new ExpectedResults();
        public ExpectedResults TestResults
        {
            get { return testResults; }
            set { testResults = value; }
        }

        ModelCheckResultList checkResult = null;

        List<string> lastErrors = null; 
        public List<string> LastErrors
        {
            get { return lastErrors; }
            set { lastErrors = value; }
        }

        List<string> messages = null;
        public List<string> Messages
        {
            get { return messages; }
            set { messages = value; }
        }

        ResultRules resultRules = new ResultRules();
        public ResultRules ResultRules
        {
            get { return resultRules; }
            set { resultRules = value; }
        }

        TestCaseTableStorage testCaseStorage = new TestCaseTableStorage();
        public TestCaseTableStorage TestCaseStorage
        {
            get { return testCaseStorage; }
            set { testCaseStorage = value; }
        }

        public TestModel()
        {

        }

        /// <summary>
        /// The main function to generate test cases using one of the test case generators
        /// </summary>
        /// <param name="orderOfCombinations">The desired order depth. At the moment this is only supported by Microsoft PICT generator</param>
        /// <param name="cleanBuild">If true, all results are dropped.</param>
        /// <returns>false, if any exceptions occured during generation</returns>
        public bool GenerateTestCases(int orderOfCombinations, bool cleanBuild)
        {
            lastErrors = new List<string>();
            messages = new List<string>();

            bool result = false;
            // save an existing test case table to temporary to reassign the results after regenerating
            if (!(testCases == null))
                testCasesTemp = testCases.Copy();

            try
            {
                // get the generator depending on global settings
                // and initialize the object with the needed data from the test model
                generator = GeneratorFactory.GetGenerator();
                generator.ModelParameters = testParameters;
                generator.OrderOfCombination = orderOfCombinations;
                generator.Calculate();

                // build the new table
                testCases = new TestCaseDataTable();
                testCases.Param = testParameters;
                testCases.Results = testResults;
                testCases.BuildTestCaseTable(generator.ParameterCombinations);
                testCases.ProcessNullConditions(testParameters);
                testCases.ResyncResultColumns();

                // try to reassign the expected results (if they are results ...)
                if ((!cleanBuild) && (testCasesTemp != null))
                    ReassignResults();

                result = true;
            }
            catch (Exception ex)
            {
                // in case of error restore the original data
                if (testCasesTemp != null)
                    testCases = testCasesTemp.Copy();
                lastErrors.Add("Fehler beim Generieren der Testflle:");
                lastErrors.Add(ex.Message);
                if (ex.InnerException != null)
                    lastErrors.Add(ex.InnerException.Message);
                result = false;
            }

            messages.Add("Generation of test cases with a combinatorial depth of " + orderOfCombinations.ToString()
                + " completed.");
            if (cleanBuild)
                messages.Add("Option: 'CleanBuild'. All results discarded!");

            return result;
        }

        /// <summary>
        /// Has to be called after loading a test model from xml to synchronize the member objects
        /// </summary>
        public void AfterLoad()
        {
            if (testCases != null)
            {
                testCases.Results = testResults;
                testCases.Param = testParameters;
            }
        }

 
        /// <summary>
        /// reassign the results after regeneration
        /// </summary>
        public void ReassignResults()
        {
            if (testCasesTemp != null)
            {
                List<string> paramCols = testCasesTemp.GetParamColumns();
                foreach (DataRow dr in testCases.Rows)
                {
                    string key = "";
                    foreach (string col in paramCols)
                    {
                        key += dr[col];
                    }
                    // search for coresponding parameter constallation in old table
                    DataRow drOld = testCasesTemp.LocateRowByKey(key);
                    // copy the result data
                    if (!(drOld == null))
                    {
                        List<string> resultCols = testCasesTemp.GetResultColumns();
                        foreach (string c in resultCols)
                        {
                            dr[c] = drOld[c];
                        }
                    }
                }
            }
        }


        /// <summary>
        /// Checks the whole model for certaing specifics
        /// Uses further helper functions for the real checking stuff
        /// </summary>
        /// <returns></returns>
        public ModelCheckResultList CheckModel()
        {
            checkResult = new ModelCheckResultList();
            CheckForMissingResults();
            CheckForUnassignedResults();
            return checkResult;
        }


        /// <summary>
        /// Checks all test cases for missing results
        /// </summary>
        private void CheckForMissingResults()
        {
            List<string> resCols = testCases.GetResultColumns();
            if ((testCases != null) && (resCols.Count > 0))
            {
                foreach (DataRow dr in testCases.Rows)
                {
                    bool hasResultAssigned = false;
                    foreach (string colName in resCols)
                    {
                        if (dr[colName].GetType() != typeof(DBNull))
                        {
                            if ((bool)dr[colName] == true)
                            {
                                hasResultAssigned = true;
                                break;
                            }
                        }
                    }
                    if (!hasResultAssigned)
                    {
                        ModelCheckResult res = new ModelCheckResult();
                        res.Result = ModelCheckResult.ResultType.ResultIsMissing;
                        res.ObjectDesc = dr["TestcaseId"].ToString();
                        checkResult.Add(res);
                    }
                }
            }
        }

        /// <summary>
        /// Checks if all defined results are used somewhere in the model
        /// </summary>
        private void CheckForUnassignedResults()
        {
            List<string> resultCols = testCases.GetResultColumns();
            foreach (string col in resultCols)
            {
                bool assigned = false;
                foreach (DataRow dr in testCases.Rows)
                {
                    if (dr[col].GetType() != typeof(DBNull))
                        if ((bool)dr[col] == true)
                        {
                            assigned = true;
                            break;
                        }
                }
                if (!assigned)
                {
                    ModelCheckResult res = new ModelCheckResult();
                    res.ObjectDesc = col;
                    res.Result = ModelCheckResult.ResultType.ResultIsNeverUsedInModel;
                    checkResult.Add(res);
                }
            }
        }


        /// <summary>
        /// Doesn't work at the moment - why ever?
        /// </summary>
        /// <returns></returns>
        public bool Undo()
        {
            if (testCasesTemp != null)
            {
                testCases = testCasesTemp.Copy();
                return true;
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// Checks whether the model is ready for test case generations
        /// </summary>
        /// <returns>True if everything is fine. False if something is missing or wrong. You have to check LastErros for more information.</returns>
        public bool GenerationPossible()
        {
            bool result = false;
            lastErrors = new List<string>();

            // check number of parameters
            if (testParameters.Count > 1)
                result = true;
            else
            {
                lastErrors.Add("At least two parameters required!");
            }

            // check if each parameter has at least on value
            foreach (Parameter par in testParameters.Values)
            {
                if (par.Classes.Count == 0)
                {
                    lastErrors.Add("Parameter '" + par + "' has no equivalence classes!");
                    result = false;
                    break;
                }
            }

            return result;
        }

        /// <summary>
        /// Apply the result rules that are defined in the model to the existing test cases
        /// </summary>
        public void ApplyResultRules()
        {
            testCases.ProcessRules(resultRules);
        }

        /// <summary>
        /// This gives the last checkresults back
        /// </summary>
        /// <returns>Check Results</returns>
        public ModelCheckResultList GetCheckResults()
        {
            return checkResult;
        }

        /// <summary>
        /// Saves the current test case table to the storage dictionary
        /// </summary>
        /// <param name="nameInStorage"></param>
        public void SaveCurrentTestCasesToStorage(string nameInStorage)
        {
            testCaseStorage.Add(nameInStorage, testCases.Copy());
            //testCases.WriteXml("e:\\temp\\testcases.xml", XmlWriteMode.WriteSchema);
        }


        /// <summary>
        /// recall a copy from a stored test case table
        /// </summary>
        /// <param name="nameInStorage"></param>
        public void RecallFormTestCaseStorage(string nameInStorage)
        {
            testCases = testCaseStorage[nameInStorage].Copy();
        }
    }

}
