// TestCaseGeneratorUI
// (c)2007 Mark Bulmahn. mBulmahn@web.de
// License: GPL 2.0
// Project: TestCaseGenerator. http://www.codeplex.com/TestCaseGenerator

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using TestCaseModel;

namespace TestCaseGenerator.Controls
{
    public partial class ParameterClassesControl : UserControl
    {
        private const int NODETYPE_PARAM = 1;
        private const int NODETYPE_CLASS = 2;
        private const string NEW_CLASS = "New class";
        private const string NEW_PARAM = "New parameter";

        Parameter currentParam = null;
        EquivClass currentClass = null;
        TreeNode currentNode = null;
        TreeNode currentParamNode = null;

        Parameters parameters = null;
        public Parameters TestParameters
        {
            get { return parameters; }
            set 
            { 
                parameters = value;
                if (parameters != null)
                    Resync();
            }
        }

        bool selectionOnly = false;
        public bool SelectionOnly
        {
            get { return selectionOnly; }
            set
            {
                selectionOnly = value;
                SelectionOnlyChanged();
            }
        }

        TestModel model = null;
        public TestModel Model
        {
            get { return model; }
            set { model = value; }
        }

        public delegate void NodeChangedEventHandler(TreeNode currentNode, Parameter currentParameter, EquivClass currentClass);
        public delegate void DoubleClickEventHandler(Parameter currentParameter, EquivClass currentClass);
        public delegate void NbrOfParametersChangedEventHandler(int nbrOfParameters);

        public event NodeChangedEventHandler NodeChanged;
        public new event DoubleClickEventHandler DoubleClick;
        public event NbrOfParametersChangedEventHandler NbrOfParametersChanged;


        public ParameterClassesControl()
        {
            InitializeComponent();
            tree.LabelEdit = true;
            textExampleData.Enabled = true;
        }

        void SelectionOnlyChanged()
        {
            if (selectionOnly)
            {
                toolStrip1.Visible = false;
                tree.LabelEdit = false;
            }
            else
            {
                toolStrip1.Visible = true;
                tree.LabelEdit = true;
            }
        }

        private void Resync()
        {
            tree.Nodes.Clear();
            foreach (Parameter par in parameters.SortedParameters)
            {
                TreeNode newParameterNode = new TreeNode(par.Name);
                newParameterNode.Tag = NODETYPE_PARAM;
                newParameterNode.ImageIndex = 0;
                if (par.NullConditionExpr != null)
                    newParameterNode.ForeColor = Color.Red;
                foreach (EquivClass cl in par.Classes.SortedClasses)
                {
                    TreeNode newClassNode = new TreeNode(cl.Id);
                    newClassNode.ImageIndex = 1;
                    newClassNode.Tag = NODETYPE_CLASS;
                    newParameterNode.Nodes.Add(newClassNode);
                }
                tree.Nodes.Add(newParameterNode);
            }
            tree.ExpandAll();
        }

        private void AddNewParam()
        {
            if (!parameters.ContainsKey(NEW_PARAM))
            {
                Parameter newParameter = new Parameter();
                newParameter.Name = NEW_PARAM;
                newParameter.OrderId = parameters.GetNewOrderId();
                parameters.Add(newParameter.Name, newParameter);

                TreeNode tn = new TreeNode(newParameter.Name);
                tn.ImageIndex = 0;
                tn.Tag = NODETYPE_PARAM;
                tree.Nodes.Add(tn);
                tree.SelectedNode = tn;
                tn.BeginEdit();
                if (NbrOfParametersChanged != null)
                    NbrOfParametersChanged(parameters.Count);
            }
            else
            {
                MessageBox.Show("Parameter with Name '" + NEW_PARAM + "' allready exists!\n" +
                        "Please give another name!");
            }
        }

        private void AddNewClass()
        {
            if (currentParam != null)
            {
                if (!parameters[currentParam.Name].Classes.HasId(NEW_CLASS))
                {
                    EquivClass newClass = new EquivClass();
                    newClass.Id = NEW_CLASS;
                    newClass.OrderId = parameters[currentParam.Name].Classes.GetNewOrderId();
                    parameters[currentParam.Name].Classes.Add(newClass);

                    TreeNode tn = new TreeNode(newClass.Id);
                    tn.ImageIndex = 1;
                    tn.Tag = NODETYPE_CLASS;
                    currentParamNode.Nodes.Add(tn);
                    tree.SelectedNode = tn;
                    tn.BeginEdit();
                }
                else
                {
                    MessageBox.Show("Class with name '" + NEW_CLASS + "' allready exists!\n" +
                             "Please give another name!");
                }
            }
            else
            {
                MessageBox.Show("quivalenzklassen knnen nur angelegt werden, wenn ein Parameter existiert\nund ausgewhlt ist!");
            }
        }

        private void NodeEdited(NodeLabelEditEventArgs e)
        {
            if ((int)e.Node.Tag == NODETYPE_PARAM)
            {
                if (!parameters.ContainsKey(e.Label))
                {
                    // save the old parameter and remove it from the dictionary
                    Parameter tempPar = parameters[e.Node.Text];
                    parameters.Remove(e.Node.Text);
                    // and add the edited parameter
                    tempPar.Name = e.Label;
                    parameters.Add(tempPar.Name, tempPar);
                }
                else
                {
                    MessageBox.Show("Parameter mit Namen '" + e.Label + "' existiert bereits!\n" +
                        "Bitte vergeben Sie einen anderen Namen!");
                    e.CancelEdit = true;
                }
            }

            if ((int)e.Node.Tag == NODETYPE_CLASS)
            {
                if (!parameters[currentParam.Name].Classes.HasId(e.Label))
                {
                    Parameter par = parameters[e.Node.Parent.Text];
                    for (int i = 0; i < par.Classes.Count; i++)
                    {
                        if (par.Classes[i].Id == e.Node.Text)
                        {
                            par.Classes[i].Id = e.Label;
                            break;
                        }
                    }
                }
                else
                {
                    MessageBox.Show("Klasse mit Namen '" + e.Label + "' existiert bereits!\n" +
                        "Bitte vergeben Sie einen anderen Namen!");
                    e.CancelEdit = true;
                }
            }
        }


        private void NodeSelected(TreeViewEventArgs e)
        {
            currentNode = e.Node;
            if ((int)currentNode.Tag == NODETYPE_CLASS)
            {
                //currentClass = new EquivClass(currentNode.Text);
                currentParam = parameters[currentNode.Parent.Text];
                currentClass = currentParam.Classes.GetById(currentNode.Text);
                e.Node.SelectedImageIndex = 1;
                SyncExampleData();
            }
            if ((int)currentNode.Tag == NODETYPE_PARAM)
            {
                currentParam = parameters[currentNode.Text];
                currentClass = null;
                currentParamNode = e.Node;
                e.Node.SelectedImageIndex = 0;

                // no example data bindinng
                UnbindExampleData();
            }
            if (NodeChanged != null)
                NodeChanged(currentNode, currentParam, currentClass);

            
        }

        private void SyncExampleData()
        {
            if (currentClass != null)
            {
                lblExampleData.Text = "Example Data for " + currentClass.Id + ":";
                textExampleData.DataBindings.Clear();
                textExampleData.DataBindings.Add(new Binding("Text", currentClass, "ExampleData"));
                textExampleData.Enabled = true;
            }
            else
            {
                UnbindExampleData();
            }
        }

        private void UnbindExampleData()
        {
            lblExampleData.Text = "Example Data: ";
            textExampleData.DataBindings.Clear();
            textExampleData.Text = "";
            textExampleData.Enabled = false;
        }

        void DeleteNode()
        {
            if (currentNode != null)
            {
                if ((int)currentNode.Tag == NODETYPE_PARAM)
                {
                    if (MessageBox.Show("Soll der Parameter '" + currentParam.Name + "' gelscht werden?",
                        "Parameter lschen", MessageBoxButtons.YesNo) == DialogResult.Yes)
                    {
                        parameters.Remove(currentParam.Name);
                        currentNode.Remove();

                        if (NbrOfParametersChanged != null)
                            NbrOfParametersChanged(parameters.Count);
                    }
                }

                if ((int)currentNode.Tag == NODETYPE_CLASS)
                {
                    if (MessageBox.Show("Soll die quivalenzklasse '" + currentClass + "' wirklich gelscht werden?",
                        "quivalenzklasse lschen", MessageBoxButtons.YesNo) == DialogResult.Yes)
                    {
                        currentParam.Classes.RemoveById(currentClass.Id);
                        currentNode.Remove();
                    }
                }
            }
        }

        
        private void EditNullCondition()
        {
            if ((currentNode != null) && ((int)currentNode.Tag == NODETYPE_PARAM))
            {
                ResultRule oldNullCondExpr = null;
                FrmRuleExpressionEditor expEditor = new FrmRuleExpressionEditor();
                if (currentParam.NullConditionExpr == null)
                {
                    currentParam.NullConditionExpr = new ResultRule();
                    currentParam.NullConditionExpr.RuleType = ResultRule.ResultRuleType.NullConditionRule;
                    currentParam.NullConditionExpr.RuleName = "NullCond" + currentParam.Name;
                    // for backwards compatibility only
                    if (currentParam.NullCondition != "")
                        currentParam.NullConditionExpr.RuleExpression = currentParam.NullCondition;
                }
                else
                {
                    oldNullCondExpr = new ResultRule(currentParam.NullConditionExpr);
                }
                expEditor.Rule = currentParam.NullConditionExpr;
                expEditor.Model = model;
                expEditor.Resync();

                if (expEditor.ShowDialog() == DialogResult.Cancel)
                    currentParam.NullConditionExpr = oldNullCondExpr;
            }
            else
            {
                MessageBox.Show("You have to select a parameter to create a null condition!");
            }
            Resync();
        }

        private void RemoveNullCondition()
        {
            if ((currentNode != null) && ((int)currentNode.Tag == NODETYPE_PARAM) && (currentParam.NullConditionExpr != null))
            {
                if (MessageBox.Show("Really remove the null condition?", "Remove null condition", MessageBoxButtons.YesNo) == DialogResult.Yes)
                {
                    currentParam.NullCondition = "";
                    currentParam.NullConditionExpr = null;
                    Resync();
                }
            }
            else
            {
                MessageBox.Show("You have to select a parameter node with a null condition\nto remove the null condition from it!");
            }
        }


        private void MoveNodeUp()
        {
            if (currentNode != null)
            {
                if ((int)currentNode.Tag == NODETYPE_PARAM)
                {
                    parameters.MoveUp(currentParam);
                }

                if ((int)currentNode.Tag == NODETYPE_CLASS)
                {
                    currentParam.Classes.MoveUp(currentClass);
                }
                Resync();
                tree.SelectedNode = currentNode;
            }
        }


        private void MoveNodeDown()
        {
            if (currentNode != null)
            {
                if ((int)currentNode.Tag == NODETYPE_PARAM)
                {
                    parameters.MoveDown(currentParam);
                }

                if ((int)currentNode.Tag == NODETYPE_CLASS)
                {
                    currentParam.Classes.MoveDown(currentClass);
                }
                Resync();
            }
        }



        private void ParameterClassesControl_Load(object sender, EventArgs e)
        {

        }

        private void butNewParam_Click(object sender, EventArgs e)
        {
            AddNewParam();
        }

        private void butNewClass_Click(object sender, EventArgs e)
        {
            AddNewClass();
        }

        private void tree_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (e.Node != null)
            {
                NodeSelected(e);
            }
        }

        private void tree_AfterLabelEdit(object sender, NodeLabelEditEventArgs e)
        {
            if (e.Label != null)
            {
                NodeEdited(e);
            }
        }

        private void tree_BeforeLabelEdit(object sender, NodeLabelEditEventArgs e)
        {
            //if ((int)e.Node.Tag == NODETYPE_PARAM)
            //    e.CancelEdit = true;
        }

        private void butDelete_Click(object sender, EventArgs e)
        {
            DeleteNode();
        }

        private void tree_DoubleClick(object sender, EventArgs e)
        {
            if (DoubleClick != null)
            {
                if ((int)currentNode.Tag == NODETYPE_PARAM)
                {
                    DoubleClick(currentParam, null);
                }

                if ((int)currentNode.Tag == NODETYPE_CLASS)
                {
                    DoubleClick(currentParam, currentClass);
                }
                tree.ExpandAll();
            }
        }

        private void nullConditionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            EditNullCondition();
        }

        private void removeNullConditionFromParameterToolStripMenuItem_Click(object sender, EventArgs e)
        {
            RemoveNullCondition();
        }

        private void deleteValueToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Not yet implemented :-(");
        }

        private void butDown_Click(object sender, EventArgs e)
        {
            MoveNodeDown();
        }

        private void butUp_Click(object sender, EventArgs e)
        {
            MoveNodeUp();
        }
    }
}
