// TestCaseGeneratorUI
// (c)2007 Mark Bulmahn. mBulmahn@web.de
// License: GPL 2.0
// Project: TestCaseGenerator. http://www.codeplex.com/TestCaseGenerator

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using TestCaseModel;
using System.IO;
using Org.Mentalis.Utilities;

namespace TestCaseGenerator
{
    /// <summary>
    /// The controller logic for the main form
    /// Actually this is where the whole ui logic is located. Not pretty, but ...
    /// </summary>
    public partial class FrmMain : Form
    {
        TestModel model = new TestModel();
        
        string currentFileName = "";
        public string CurrentFileName
        {
            get { return currentFileName; }
            set
            {
                currentFileName = value;
                this.Text = currentFileName + " - TestCaseGenerator";
                if (currentFileName == "")
                {
                    butSaveModel.Enabled = false;
                }
                else
                {
                    butSaveModel.Enabled = true;
                }
            }
        }

        private void AddNewParam(string t)
        {
            string pname = t;
            if (!model.TestParameters.ContainsKey(pname))
            {
                Parameter p = new Parameter();
                p.Name = pname;
                model.TestParameters.Add(pname, p);
                Resync();
            }
            else
            {
                MessageBox.Show("Parameter does not exist!");
            }
        }


        private void SelectedClassChanged()
        {
        }


        private void Resync()
        {
            model.GeneratorOptions.CombinationDepth = model.TestParameters.Values.Count;


            resultRuleExpressionUI1.Model = model;
            resultsEditorControl1.Results = model.TestResults;
            parameterClassesControl1.Model = model;
        }

        private void LoadModel()
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                DoLoadModel(openFileDialog1.FileName);
            }
        }

        private void DoLoadModel(string fn)
        {
            model = TestModel.CreateFromXml(fn);
            model.AfterLoad();
            Resync();
            AssignDataSourceToGrid(model.TestCases);
            rtfDescription.Rtf = model.TestDescription;
            CurrentFileName = fn;
            parameterClassesControl1.TestParameters = model.TestParameters;
            RefreshButLoadTcTable();
            this.generatorOptionsUi1.CurrentOptions = model.GeneratorOptions;
        }

        private void AutoLoadModel()
        {
            if (AppCore.AppGlobal.Args.Length > 0)
            {
                string fn = AppCore.AppGlobal.Args[0];
                if (File.Exists(fn))
                {
                    DoLoadModel(fn);
                }
            }
        }

        private void SaveModelAs()
        {
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                string fn = saveFileDialog1.FileName;
                if ((fn.Length < 10) || (!fn.EndsWith(".TestModel")))
                {
                    fn += ".TestModel";
                }
                DoSaveModel(fn);
                CurrentFileName = fn;
            }
        }

        private void SaveModel()
        {
            DoSaveModel(currentFileName);
        }

        private void DoSaveModel(string fn)
        {
            model.TestDescription = rtfDescription.Rtf;
            model.SaveAsXml(fn);
        }

        private void QuitApp()
        {
            
        }

        private void ShowAbout()
        {
            FrmAbout f = new FrmAbout();
            f.ShowDialog();
        }

        private void NewDoc()
        {
            if (MessageBox.Show("All unsaved data will be lost.\nDo you really want to create a new model?",
                "New Model?", MessageBoxButtons.OKCancel) == DialogResult.OK)
            {
                model = new TestModel();
                InitAfterNew();
            }
        }

        private void InitAfterNew()
        {
            Resync();
            AssignDataSourceToGrid(model.TestCases);
            rtfDescription.Rtf = model.TestDescription;
            CurrentFileName = "";
            parameterClassesControl1.TestParameters = model.TestParameters;
            resultsEditorControl1.Results = model.TestResults;
            butSaveTcTable.Enabled = false;
            butLoadTcTable.Enabled = false;
            generatorOptionsUi1.CurrentOptions = model.GeneratorOptions;

            switch (GlobalSettings.GeneratorToUse)
            {
                case GlobalSettings.TcGenerator.MSPICT:
                    ChooseGenPict();
                    break;
                case GlobalSettings.TcGenerator.SIMPLE:
                    ChooseGenBuiltIn();
                    break;
            }
        }

        private void GenTestCases(bool cleanBuild)
        {
            if (model.GenerationPossible())
            {
                //if (cleanBuild)
                //{
                //    if (MessageBox.Show("Attention! All assigned results in the test cases\nwill be deleted by this operation.\nContinue?",
                //        "Generate", MessageBoxButtons.YesNo) == DialogResult.No)
                //        return;
                //}

                int order = model.GeneratorOptions.CombinationDepth;
                if (order < 2)
                {
                    MessageBox.Show("Combinatorial depth smaller than two is not allowed!");
                    return;
                }

                if (order > model.TestParameters.Values.Count)
                {
                    MessageBox.Show("The combinatorial depth must not be greater\nthan the number of parameters!");
                    return;
                }

                if (model.GenerateTestCases())
                {
                    AssignDataSourceToGrid(model.TestCases);
                }
                else
                {
                    textProto.Lines = model.LastErrors.ToArray();
                    SetMessage("Error in generation!");
                    MessageBox.Show("Errors occured in generation!\n" +
                        "Last results are restored!");
                }
            }
            else
            {
                MessageBox.Show("Unable to generate model! Please look at the messages for possible reasons.");
                textProto.Lines = model.LastErrors.ToArray();
            }
        }

        private void AssignDataSourceToGrid(DataTable dt)
        {
            if (dgvResult.Columns.Count > 0)
                dgvResult.Columns.Clear();
            dgvResult.DataSource = dt;
            //dgvResult.Columns[dgvResult.Columns.Count - 1].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;

            // make all result columns editable
            foreach (DataGridViewColumn gridCol in dgvResult.Columns)
            {
                string dataColName = gridCol.DataPropertyName;

                if (dataColName == "TestcaseId")
                    gridCol.Frozen = true;

                if ((string)model.TestCases.Columns[dataColName].ExtendedProperties[TestModel.COLTYPE]
                    == TestModel.COLTYPE_RESULT)
                {
                    gridCol.ReadOnly = false;
                }
                else
                {
                    gridCol.ReadOnly = true;
                }
            }

            if (dgvResult.Rows.Count > 0)
            {
                butSaveTcTable.Enabled = true;
                butLoadTcTable.Enabled = true;
            }

            // hide the internally used colum for model checking purposes
            if (dgvResult.Columns.Contains(TestCaseDataTable.COL_RESULTSTATE))
                dgvResult.Columns[TestCaseDataTable.COL_RESULTSTATE].Visible = false;
        }

        private void ResultAdded(string inputText)
        {
            if (inputText.Trim() != "")
            {
                model.TestResults.Add(new ExpectedResult(inputText));
                model.TestCases.ResyncResultColumns();
            }
        }

        private void ResultRemoved(string inputText)
        {
            foreach (ExpectedResult res in model.TestResults)
            {
                if (res.Id == inputText)
                {
                    model.TestResults.Remove(res);
                    model.TestCases.ResyncResultColumns();
                    break;
                }
            }
        }

        private void CheckModel()
        {
            if ((model.TestCases != null) && (model.TestCases.Rows.Count > 0))
            {
                string[] checkResult = model.CheckModel().GetAsText().ToArray();
                textProto.Lines = checkResult;
                if (checkResult.Length != 0)
                {
                    SetMessage("Check completed. " + checkResult.Length.ToString() + " Message. Please check!");
                    MarkMissingResults();
                }
                else
                    SetMessage("Check completed without any result.");
            }
            else
            {
                MessageBox.Show("Model can be checked after test case generation!");
            }
        }

        /// <summary>
        /// Mark all rows with missing results red.
        /// </summary>
        private void MarkMissingResults()
        {
            // first set all rows to white
            foreach (DataGridViewRow dgvRow in dgvResult.Rows)
            {
                dgvRow.DefaultCellStyle.BackColor = Color.White;
            }

            // then mark the ones with missing results with red background color
            foreach (ModelCheckResult res in model.GetCheckResults())
            {
                if (res.Result == ModelCheckResult.ResultType.ResultIsMissing)
                {
                    string tcId = res.ObjectDesc;
                    foreach (DataGridViewRow dgvRow in dgvResult.Rows)
                    {
                        if ((int)dgvRow.Cells[0].Value == Convert.ToInt32(tcId))
                        {
                            dgvRow.DefaultCellStyle.BackColor = Color.OrangeRed;
                        }
                    }
                }
            }
        }

        private void SetMessage(string text)
        {
            toolStripStatusLabel1.Text = text;
        }

        private void UndoGen()
        {
            if (!model.Undo())
                MessageBox.Show("Undo war nicht mglich!");
        }

        private void ExportTestCases(TestCaseExporter.ExportType t)
        {
            TestCaseExporter exporter = new TestCaseExporter();
            exporter.TypeOfExport = t;

            switch (t)
            {
                case TestCaseExporter.ExportType.CSV:
                    saveFileDialogExport.FilterIndex = 1;
                    if (saveFileDialogExport.ShowDialog() == DialogResult.OK)
                    {
                        exporter.FileName = saveFileDialogExport.FileName;
                    }
                    break;

                case TestCaseExporter.ExportType.EXCEL:
                    saveFileDialogExport.FilterIndex = 2;
                    if (saveFileDialogExport.ShowDialog() == DialogResult.OK)
                    {
                        exporter.FileName = saveFileDialogExport.FileName;
                        exporter.ExportData(model.TestCases);
                    }
                    break;

                case TestCaseExporter.ExportType.SQLCE:
                    FrmExpSqlCe expFormCe = new FrmExpSqlCe();
                    if (expFormCe.ShowDialog() == DialogResult.OK)
                    {
                        exporter.ConnectionString = expFormCe.ConnectionString;
                        exporter.DestTableName = expFormCe.DestTableName;
                        exporter.ExportData(model.TestCases);
                    }
                    break;

                case TestCaseExporter.ExportType.MSSQL:
                    FrmExpMssql expForm = new FrmExpMssql();
                    if (expForm.ShowDialog() == DialogResult.OK)
                    {
                        exporter.ConnectionString = expForm.ConnectionString;
                        exporter.DestTableName = expForm.DestTableName;
                        exporter.ExportData(model.TestCases);
                    }
                    break;

                case TestCaseExporter.ExportType.XML:
                case TestCaseExporter.ExportType.XMLWithHeader:
                    SaveFileDialog sdlg = new SaveFileDialog();
                    sdlg.Filter = "*.XML|XML-File";
                    if (sdlg.ShowDialog() == DialogResult.OK)
                    {
                        exporter.FileName = sdlg.FileName;
                        exporter.ExportData(model.TestCases);
                    }
                    break;

                default:
                    break;
            }
        }

        void ApplyRules()
        {
            model.ApplyResultRules();
        }

        private void NumberOfParametersChanged(int nbrOfParameters)
        {
            model.GeneratorOptions.CombinationDepth = model.TestParameters.Count;
        }

        private void ChangeSettings()
        {
            FrmSettings settings = new FrmSettings();
            settings.ShowDialog();
        }

        private void ChooseGenPict()
        {
            GeneratorIsNow(GlobalSettings.TcGenerator.MSPICT);
        }

        private void ChooseGenBuiltIn()
        {
            GeneratorIsNow(GlobalSettings.TcGenerator.SIMPLE);
        }

        private void GeneratorIsNow(GlobalSettings.TcGenerator tcGenerator)
        {
            Properties.Settings.Default.GeneratorToUse = (int)tcGenerator;
            Properties.Settings.Default.Save();
            TestCaseModel.GlobalSettings.GeneratorToUse = tcGenerator;
        }

        private void SaveTcTable()
        {
            FrmInputBox inp = new FrmInputBox("Save test cases", "File name?");
            if (inp.ShowDialog() == DialogResult.OK)
            {
                model.SaveCurrentTestCasesToStorage(inp.InputText);
                RefreshButLoadTcTable();
            }
        }

        private void RefreshButLoadTcTable()
        {
            if (model.TestCaseStorage.Values.Count > 0)
            {
                butLoadTcTable.DropDownItems.Clear();
                foreach (string key in model.TestCaseStorage.Keys)
                {
                    butLoadTcTable.DropDownItems.Add(key);
                }
                foreach (ToolStripDropDownItem item in butLoadTcTable.DropDownItems)
                {
                    item.Click += new EventHandler(tcStorageitem_Click);
                }
            }
        }

        private void tcStorageitem_Click(Object sender, EventArgs e)
        {
            ToolStripDropDownItem item = (ToolStripDropDownItem)sender;
            if (MessageBox.Show("Attention!\nThe current test case table will be dropped!\nContinue?", 
                "Load test case table?", MessageBoxButtons.OKCancel) == DialogResult.OK)
            {
                model.RecallFormTestCaseStorage(item.Text);
                dgvResult.DataSource = model.TestCases;
            }
        }

        private void ExpTcTableAsHtml()
        {
            string fn = "c:\\temp\\tc.html";
            model.TestCases.ExportAsHtml(fn);
            MessageBox.Show(fn);
        }

        private void FillInExampleData()
        {
            model.TestCases.ReplaceWithExampleData();
        }

        private void DoRegisterFileType()
        {
            if (MessageBox.Show("This will register the file type .TestModel with\nthis application. Is this what you want to do?",
                "Register file type", MessageBoxButtons.OKCancel, MessageBoxIcon.Warning) == DialogResult.OK)
            {
                string pathToApp = Application.ExecutablePath;
                FileAssociation FA = new FileAssociation();
                FA.Extension = "TestModel";
                FA.ContentType = "application/TestCaseGenerator";
                FA.FullName = "Test case model definition";
                FA.ProperName = "Test case model definition";
                FA.AddCommand("open", "\"" + pathToApp + "\" %1");
                FA.Create();
            }
        }
    }
}
